<?php

namespace Drupal\usasearch;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;
use Drupal\Core\Session\AnonymousUserSession;
use Drupal\taxonomy\Entity\Term;
use Drupal\Core\Render\RenderableInterface;

/**
 * Provides a document in the format to be used as an i14y document.
 */
class UsaSearchDocument {

  /**
   * A unique document ID.
   */
  public $document_id;
  /**
   * Document title.
   */
  public $title;
  /**
   * Document description.
   */
  public $description;
  /**
   * Document content.
   */
  public $content;
  /**
   * Document link URL.
   */
  public $path;
  /**
   * When the document was created (such as ‘2013-02-27T10:00:00Z’).
   */
  public $created;
  /**
   * When the document was updated (such as ‘2013-02-27T10:00:00Z’).
   */
  public $changed;
  /**
   * Two letter language locale.
   */
  public $language;
  /**
   * Whether to promote the document in the relevance ranking.
   */
  public $promote = FALSE;
  /**
   * Comma-separated list of case-insentitive tags.
   */
  public $tags;

  private $node = NULL;
  private $status = 0;

  /**
   * Constructs a new UsaSearchDocument.
   *
   * @param \Drupal\node\NodeInterface $node
   *   A node object to convert.
   * @param string $force
   *   Force document creation, ignoring node access.
   */
  public function __construct(NodeInterface $node, $force = FALSE) {
    // @Todo: use dependency injection for Services used
    $this->node = $node;
    $this->status = $this->node->isPublished();
    $anon = new AnonymousUserSession();
    $access = $node->access('view', new AnonymousUserSession());
    if (($this->status && $access) || $force == TRUE) { //this->node->access(new AnonymousUserSession())
      $config = \Drupal::config('usasearch.settings');
      $this->document_id = $this->node->id();
      $this->title = $this->node->getTitle();
      $this->path = Url::fromUri('entity:node/' . $this->node->id(), array('absolute' => TRUE))->toString();
      if ($alternate_baseurl = $config->get('alternate_baseurl')) {
        $fullurl = parse_url($this->path);
        $this->path = $alternate_baseurl . $fullurl['path'];
      }
      $this->created = \Drupal::service('date.formatter')->format($this->node->getCreatedTime(), 'custom', 'c');
      $this->language = $this->node->language()->getId();
      // Get the view mode to be used for the description.
      $description_view_mode = $config->get('description_view_mode');
      // @todo: replace entityManager (depreciated).
      $description_view = \Drupal::entityManager()->getViewBuilder('node')->view($this->node, $description_view_mode);
      $this->description = \Drupal::service('renderer')->renderPlain($description_view);
      $view = \Drupal::entityManager()->getViewBuilder('node')->view($this->node, 'full');
      $this->content = \Drupal::service('renderer')->renderPlain($view);
      $this->promote = $this->node->isPromoted() ? TRUE : FALSE;
      $this->tags = $this->getTerms();
      $this->changed = \Drupal::service('date.formatter')->format($this->node->getChangedTime(), 'custom', 'c');
    }
  }

  /**
   * Get a list of Taxonomy Terms from the given Node.
   */
  public function getTerms() {
    $tids = $terms = array();
    if ($this->node) {
      $field_definitions = $this->node->getFieldDefinitions();
      foreach ($field_definitions as $field_definition) {
        if ($field_definition->getType() == 'entity_reference' && $field_definition->getSetting('target_type') == 'taxonomy_term') {
          $field_name = $field_definition->getName();
          $field_values = $this->node->get($field_name)->getValue();
          foreach ($field_values as $value) {
            if (isset($value['target_id'])) {
              $tids[] .= $value['target_id'];
            }
          }
        }
      }
      $taxonomy_terms = Term::loadMultiple($tids);
      foreach ($taxonomy_terms as $term) {
        // @todo: make terms keyed by vocabulary (bundle)?
        $terms[] .= $term->getName();
      }
    }
    // Return comma-separated tags.
    return implode(',', array_unique($terms));
  }

  /**
   * Check required fields for a valid UsaSearchDocument.
   */
  public function hasRequiredFields() {
    return ($this->document_id && $this->title && $this->path && $this->created) ? TRUE : FALSE;
  }

  /**
   * Return document status.
   */
  public function getStatus() {
    return $this->status;
  }

  /**
   * Return an array of public properties.
   */
  public function getRawData() {
    $public_properties = create_function('$obj', 'return get_object_vars($obj);');
    return $public_properties($this);
  }

  /**
   * Update UsaSearchDocument with an array of raw data.
   *
   * @param array $data
   *   The raw data.
   */
  public function setRawData($data) {
    foreach ($data as $key => $value) {
      if (property_exists($this, $key)) {
        $this->$key = $value;
      }
    }
    return $this;
  }

  /**
   * Return document as JSON string.
   */
  public function getJson() {
    $json = array_filter($this->getRawData(), 'strlen');
      return json_encode($json);
  }

  /**
   * Return document as JSON string.
   */
  public function toString() {
    return $this->getJson();
  }

}
